class GoogleCloudStorage

    # @param [string] project_name
    # @param [string] app_name
    # @param [string] credentials_path
    def initialize(project_name, app_name, credentials_path)
        @project_name = project_name
        @app_name = app_name
        @credentials = credentials_path

        @storage = Gcloud.storage @project_name, @credentials
        @bucket = get_bucket
    end

    def get_bucket_name
        "#{@project_name}_#{@app_name}"
    end

    private
    # @return [Bucket]
    def get_bucket
        buckets = @storage.buckets
        bucket_names = []
        buckets.each do |bucket|
            if bucket.name == get_bucket_name
                return bucket
            end
            bucket_names.push(bucket.name)
        end

        puts bucket_names

        @storage.create_bucket get_bucket_name, retries: 3
    end

    public
    # @param [string] filepath
    # @param [string] storage_path
    # @return [File]
    def put_file(filepath, storage_path)
        puts "Putting #{filepath} to #{storage_path} on Google Cloud Storage"
        @bucket.create_file filepath, storage_path
    end

    public
    # @param [string] existing_filepath
    # @param [string] new_filepath
    # @return [File]
    def copy_file(existing_filepath, new_filepath)
        existing_file = @bucket.find_file existing_filepath

        puts "Copying #{existing_filepath} to #{new_filepath} on Google Cloud Storage"
        existing_file.copy new_filepath
    end

end

require 'gcloud/storage'